// Global variables
var chimes = [];
var chimeWidth = 30;
var mouseLoc = [];
var direction = 0;
var mouseVelocity = 0;
var air = 0.95;
var swingPos = 0;
var pitch = [];

function preload() {

    // Load in the sounds of the chimes

    // These would have been used to embed on WP, however I ran into trouble when posting
    /*
    c1 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/c1.wav");
    d1 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/d1.wav");
    e1 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/e1.wav");
    g1 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/g1.wav");
    a1 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/a1.wav");
    c2 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/c2.wav");
    d2 = loadSound("https://courses.ideate.cmu.edu/15-104/f2019/wp-content/uploads/2019/12/d2.wav");
    */

    c1 = loadSound("c1.wav");
    d1 = loadSound("d1.wav");
    e1 = loadSound("e1.wav");
    g1 = loadSound("g1.wav");
    a1 = loadSound("a1.wav");
    c2 = loadSound("c2.wav");
    d2 = loadSound("d2.wav");

    // Stores the pitches of the chimes in an array
    pitch = [c1, d1, e1, g1, a1, c2, d2];
}

function soundSetup() {
    // Setting volumes

    c1.setVolume(0.25);
    d1.setVolume(0.25);
    e1.setVolume(0.25);
    g1.setVolume(0.25);
    a1.setVolume(0.25);
    c2.setVolume(0.25);
    d2.setVolume(0.25);

}

function setup() {
    createCanvas(400, 400);
    useSound();

    // Push the indivual chimes on to an array
    for(i = 0; i < 7; i ++) {
        chimes.push(makeChime(50 + i * (300 / 7), 300 - i * 15));
    }
}

function draw() {

    // This area uses an array to store the direction that the mouse is moving
    mouseLoc.push(mouseX);
    if(mouseLoc.length > 2){
        mouseLoc.shift();
    }

    if(mouseLoc[0] > mouseLoc[1]){
        direction = -1;
    } else
        if(mouseLoc[0] < mouseLoc[1]){
            direction = 1;
        }

    // mouseVelocity stores how fast the mouse is moving to adjust how fast/far the chimes move
    mouseVelocity = map(abs(mouseLoc[0] - mouseLoc[1]), 0, 50, 0, 7);

    // This part draws the chimes
    
    background(250, 230, 150);
    

    for(i = 0; i < chimes.length; i ++){
        var c = chimes[i];
        c.forces();
        
        c.x = c.x + c.vx;
        c.vx = c.vx * air;

        // These if statements account for chimes reacting to hitting eachother.
        // When chimes hit eachother, their velocity is stopped and a bit of it
        // is transfered to the chime that it hit.
        // This also triggers the audio effect.
        if(i !== 6 && chimes[i].x + chimeWidth >= chimes[i + 1].x){
            while(chimes[i + 1].x <= c.x + chimeWidth){
                chimes[i + 1].x ++;
            }
            chimes[i + 1].vx = chimes[i + 1].vx + c.vx / 4;
            c.vx = c.vx * 0.8;
            pitch[i].play();
        }

        if(i !== 0 && chimes[i].x <= chimes[i - 1].x + chimeWidth){
            while(chimes[i - 1].x + chimeWidth >= c.x){
                chimes[i - 1].x --;
            }
            chimes[i - 1].vx = chimes[i - 1].vx + c.vx / 4;
            c.vx = c.vx * 0.8;
            pitch[i].play();
        }
        c.draw();     
    }
}

function drawChime() {
    //drawing the individual chimes

    noStroke();
    
    //Most of this code is dedicated to the shading on the chimes
    fill(230);
    rect(this.x, 50, chimeWidth, this.h);
    ellipse(this.x + chimeWidth / 2, 50 + this.h, chimeWidth, 15);

    fill(200);
    rect(this.x, 50, chimeWidth / 4, this.h);
    arc(this.x + chimeWidth / 4, 50 + this.h, chimeWidth / 2, 12, PI / 2, PI);

    fill(255);
    ellipse(this.x + chimeWidth / 2, 50, chimeWidth, 15);
    ellipse(this.x + chimeWidth * 0.75, 50 + this.h / 2, 5, this.h);

    fill(210);
    ellipse(this.x + chimeWidth / 2, 50, chimeWidth * 2 / 3, 8);

}

function makeChime(x, h) {
    // The individual characteristics of each chime

    // vx refers to velocity, natX is the starting position
    return {x: x, h: h, 
        draw: drawChime,
        vx: 0,
        forces: applyForcestoChimes,
        natX: x}
        
}

function applyForcestoChimes() {
    // Applies the forces that make the chimes move with the mouse

    // When the mouse swings through the chimes they will move in the swiped direction
    var accel = 0;
    
    if(mouseX >= this.x && mouseX <= this.x + chimeWidth
    && mouseY >= 50 && mouseY < this.h + 50){
        accel = mouseVelocity * direction;
    }

    this.vx = this.vx + accel;

    // variable keeps track of if the chime is currently to the right or left of
    // the starting position
    swingPos = (this.x - this.natX) / (abs(this.x - this.natX));
    
    // Chimes swing back to their starting point
    if(this.x !== this.natX){
        this.vx = (this.vx - 0.5 * swingPos);
    }  
}



